;----ATMEL BASIC deffinition--------------
.device ATmega8
.include "atmeldef.inc" 

;----deffinition der PORT's---------------
.equ     sync      =0     ;Port B
.equ     pixel     =7     ;Port D

         .cseg
         .org	0


start:

;--- initialiesiere PORTB Graybar und SYNC --
;DDxn	PORTxn	PUD	I/O 	Pull-up 	Comment
;0 	0 	X 	Input 	No 		Tri-state (Hi-Z)
;0 	1 	0 	Input 	Yes		Pxn will source current if external pulled low.
;0 	1 	1 	Input 	No 		Tri-state (Hi-Z)
;1 	0 	X 	Output 	No 		Output Low (Sink)
;1 	1 	X 	Output 	No 		Output High (Source)


;-------setze Stackpointer (045F)----------
          ldi    r17, 0x5F
          out    SPL, r17
          ldi    r17, 0x04
          out    SPH, r17
;------------------------------------------

;------setze Graybar PORTS ----------------
          ldi    r17, 0x01
          out    DDRB, r17
          ldi    r17, 0x00
          out    PORTB, r17
;------------------------------------------

;------setze VIDEO PORTS-------------------
	sbi	DDRD, pixel
	sbi	DDRB, sync
	cbi	PORTD, pixel
	cbi	PORTB, sync
;------------------------------------------



;******************************************************************************************
; Takt = 6MHz  =  6 Takte fr 1s
; Takt = 8MHz  =  8 Takte fr 1s
; Takt = 16MHz = 16 Takte fr 1s
;
; Aufbau Zeile 
; ------------
; eine Bildzeile dauert 64 s dies entspricht 384 / 512 / 1024 Takte
; sichtbares Bild 52s (6MHz=312t 8MHz=416 16MHz=832)
; + vsync impulse 4s  (6MHz=24t  8MHz=32  16MHz=64 )
; + Blackscreen 8s    (6MHz=48t  8MHz=64t 16MHz=128)
; 
; Bildaufbau
; ----------
; Das Fernsehbild besteht aus 2 Halbbildern (interlace) wegen der Flackerunterdrckung
; 1 Halbbild hat 312 Zeilen
;
;
;******************************************************************************************

video:    
;----start Videosignalerzeugung------------------------------------
	rcall  vsync


	ldi    r18, 0x8e           ;426 takte ( halbe Zeile)
sh21:	dec    r18                 
	brne   sh21
	rcall  hsync

	ldi	r17, 0x03	;---------- 128 Zeilen schwarz
pau1:	dec	r17
	brne	pau1
	ldi	r18, 0x80
	rcall  linecl

	nop	;-------------------------- 53 Zeilen mit weiss in der mitte
	nop
	ldi	r18, 0x35
	rcall  linecl2

	nop	;-------------------------- 128 Zeilen schwarz
	nop
	ldi	r18, 0x80
	rcall  linecl

	rjmp	video
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++






;#####( Horizontale Synchronimpuls 187 takte)############################
; 4s sync low    (4*16 = 64  takte bei 16 MHz)
; 8s Blackscreen (8*16 = 128 takte bei 16 MHz)
; +3 vom rcall
; +4 vom return
; +180 vom programm
hsync:	cbi    PORTB, sync	;2	;sync 
	cbi    PORTD, pixel	;2
	ldi    r17, 0x15	;63
hs:	dec    r17
	brne   hs
	nop  			;1
	sbi    PORTB, sync	;2

	ldi    r17, 0x24	;108 0x29	;123	;blackscreen
hs2:	dec    r17
	brne   hs2
	nop			;1
	nop			;1
	ret			;4
;#######################################################################






;#######################################################################
shortsync:
	cbi    PORTB, sync         ;2      2s sync aus (6=12t 8=16t 16=32)
	cbi    PORTD, pixel        ;2
	ldi    r18, 0x0a           ;32
sh1:	dec    r18
	brne   sh1
        nop  
        nop  

	sbi    PORTB, sync         ;2
	ldi    r18, 0x78           ;240 
sh2:	dec    r18                 ;        30s Black (6=180t 8=240t 16=480t)
	nop
	brne   sh2

	dec    r17                 ;1
	brne   shortsync           ;2
	ldi    r17, 0x05           ;1
	ret                        ;4


vsync:    ldi    r17, 0x05           ;-----------------
          sbrs   r22, 7              ;                 |
          ldi    r17, 0x06	     ;                 |
          rcall  shortsync           ;------- 6 takte--

          sbrs   r22, 7		;------
          rjmp   opage		;      |
          cbr    r22, 7		;      |  6 takte
          nop			;      |
          nop			;      |
          nop			;-------

opager:	cbi    PORTB, sync 	;2 30s sync (6=180t 8=240t 16=480t)
	cbi    PORTD, pixel	;2
	ldi    r18, 0x78	;
sh4:	dec    r18
	nop
	brne   sh4

	sbi    PORTB, sync	;2s sync (6=12t 8=16t 16=32)
	ldi    r18, 0x0a
sh5:	dec    r18
	brne   sh5
        nop  
        nop  

	dec    r17
	brne   opager

	ldi    r17, 0x05
	sbrs   r22, 7
	ldi    r17, 0x04
	rcall  shortsync
          ret

opage:    sbr   r22, 7
          rjmp  opager
;#######################################################################




;##########( leere Zeilen )##################################
; + 13 takte vom Aufruf
; + 7 bein return vom hsync

linecl:	
	cbi    PORTD, pixel	;2

ll3:	ldi    r17, 0xcd	;820 takte
ll1:	dec    r17
	nop
	brne   ll1

	nop
	nop
	rcall  hsync		;75
	dec    r18		;1
	breq   ll4		;1-2

	ldi	r17, 0x03	;9
ll5:	dec	r17
	brne	ll5
	nop			;1
	nop			;1
	rjmp   ll3		;2

ll4:	ret
;##################################################################




;##########( Zeilen mit weiss in der Mitte )###################
; + 13 takte vom Aufruf
; + 7 bein return vom hsync

linecl2:
	cbi    PORTD, pixel	;2

lc3:
	cbi    PORTD, pixel	;2
	ldi    r17, 0x44	;272 takte
lcc1:	dec    r17
	nop
	brne   lcc1

	sbi    PORTD, pixel	;2
	ldi    r17, 0x44	;272 takte
lcc2:	dec    r17
	nop
	brne   lcc2

	cbi    PORTD, pixel	;2
	ldi    r17, 0x44	;272 takte
lcc3:	dec    r17
	nop
	brne   lcc3

	rcall  hsync		;75
	dec    r18		;1
	breq   lc4		;1-2

	ldi	r17, 0x03	;9
lcc7:	dec	r17
	brne	lcc7
	nop			;1
	nop			;1
	rjmp   lc3		;2

lc4:	ret			;4
;##################################################################


         .exit